using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;

namespace FileWatcherMT
{
    /// <summary>
    /// A Generic File Watcher Process
    /// Kerry Wong
    /// http://www.kerrywong.com
    /// 8/2007
    /// </summary>
    public class FileWatcher
    {
        private Hashtable _ht = new Hashtable();
        private FileSystemWatcher _watcher = new FileSystemWatcher();

        public void AddFileWatcher()
        {
            _watcher.Path = @"c:\temp";
            _watcher.Filter = "*.txt";

            _watcher.Created += new FileSystemEventHandler(OnFileSystemChanged);
            _watcher.Changed += new FileSystemEventHandler(OnFileSystemChanged);
            _watcher.EnableRaisingEvents = true;
        }

        public void RemoveFileWatcher()
        {
            _watcher.EnableRaisingEvents = false;
            _watcher.Created -= new FileSystemEventHandler(OnFileSystemChanged);
            _watcher.Changed -= new FileSystemEventHandler(OnFileSystemChanged);           
        }

        private void OnFileSystemChanged(object sender, FileSystemEventArgs e)
        {
            lock (this)
            {
                if (!(_ht.Contains(e.FullPath)))
                {
                    FileInfo fInfo = new FileInfo(e.FullPath);
                    _ht.Add(e.FullPath, e);
                    FileStateObject fStatus = new FileStateObject(e.FullPath, _ht);
                    Thread t = new Thread(new ThreadStart(fStatus.CheckFileStatus));
                    t.Start();
                }
            }
        }
    }

    public class FileStateObject
    {
        private string _fullPath = string.Empty;
        private long _fileSize = 0;
        private Hashtable _ht = null;

        public FileStateObject(string fullPath, Hashtable ht)
        {
            _fullPath = fullPath;
            _ht = ht;
        }

        private void ProcessFile()
        {
            //Code to process the file.
        }

        public void CheckFileStatus()
        {
            FileInfo fInfo = new FileInfo(_fullPath);

            while (_fileSize != fInfo.Length) 
            {
                _fileSize = fInfo.Length;
                Thread.Sleep(2500);
                fInfo = new FileInfo(_fullPath);
            }

            lock (this)
            {
                if (_ht.Contains(_fullPath))
                {
                    _ht.Remove(_fullPath);
                    ProcessFile();
                }
            }
        }

    }
}
